from .communication import Communication
from .serial_communication import SerialCommunication
from .web_communication import WebCommunication

__all__ = [
    'Communication',
    'SerialCommunication',
    'WebCommunication',
]
